@@ -318,7 +318,9 @@ function rule.apis()
         ,   "rule.on_clean"
         ,   "rule.on_package"
         ,   "rule.on_install"
+        ,   "rule.on_installcmd"
         ,   "rule.on_uninstall"
+        ,   "rule.on_uninstallcmd"
         ,   "rule.on_linkcmd"
         ,   "rule.on_buildcmd"
         ,   "rule.on_buildcmd_file"
@@ -334,7 +336,9 @@ function rule.apis()
         ,   "rule.before_clean"
         ,   "rule.before_package"
         ,   "rule.before_install"
+        ,   "rule.before_installcmd"
         ,   "rule.before_uninstall"
+        ,   "rule.before_uninstallcmd"
         ,   "rule.before_linkcmd"
         ,   "rule.before_buildcmd"
         ,   "rule.before_buildcmd_file"
@@ -350,7 +354,9 @@ function rule.apis()
         ,   "rule.after_clean"
         ,   "rule.after_package"
         ,   "rule.after_install"
+        ,   "rule.after_installcmd"
         ,   "rule.after_uninstall"
+        ,   "rule.after_uninstallcmd"
         ,   "rule.after_linkcmd"
         ,   "rule.after_buildcmd"
         ,   "rule.after_buildcmd_file"
