@@ -90,6 +90,14 @@ function _get_apis()
     ,   "toolchain.add_sysincludedirs"
     ,   "toolchain.add_frameworkdirs"
     }
+    apis.groups = {
+        -- target.add_xxx
+        "target.add_linkorders"
+    ,   "target.add_linkgroups"
+        -- package.add_xxx
+    ,   "package.add_linkorders"
+    ,   "package.add_linkgroups"
+    }
     apis.paths = {
         -- target.set_xxx
         "target.set_pcheader"
