@@ -81,12 +81,12 @@ function load(self)
 end
 
 -- make the strip flag
-function nf_strip(self, level, target)
+function nf_strip(self, level)
     local maps = {
         debug = "-Wl,-S"
     ,   all   = "-s"
     }
-    if self:is_plat("macosx", "iphoneos") then
+    if self:is_plat("macosx", "iphoneos", "watchos", "appletvos", "applexros") then
         maps.all = {"-Wl,-x", "-Wl,-dead_strip"}
     elseif self:is_plat("windows") then
         -- clang does not it on windows, TODO maybe we need test it for gcc
@@ -128,6 +128,8 @@ function nf_warning(self, level)
     ,   more       = "-Wall"
     ,   all        = "-Wall"
     ,   allextra   = {"-Wall", "-Wextra"}
+    ,   extra      = "-Wextra"
+    ,   pedantic   = "-Wpedantic"
     ,   everything = self:kind() == "cxx" and {"-Wall", "-Wextra", "-Weffc++"} or {"-Wall", "-Wextra"}
     ,   error      = "-Werror"
     }
@@ -211,6 +213,8 @@ function nf_language(self, stdname)
         _g.cxxmaps = {
             cxx98        = "-std=c++98"
         ,   gnuxx98      = "-std=gnu++98"
+        ,   cxx03        = "-std=c++03"
+        ,   gnuxx03      = "-std=gnu++03"
         ,   cxx11        = "-std=c++11"
         ,   gnuxx11      = "-std=gnu++11"
         ,   cxx14        = "-std=c++14"
@@ -279,7 +283,8 @@ function nf_sysincludedir(self, dir)
 end
 
 -- make the force include flag
-function nf_forceinclude(self, headerfile, target)
+function nf_forceinclude(self, headerfile, opt)
+    local target = opt.target
     local sourcekinds = target and target:extraconf("forceincludes", headerfile, "sourcekinds")
     if not sourcekinds or table.contains(table.wrap(sourcekinds), self:kind()) then
         return {"-include", headerfile}
@@ -288,7 +293,9 @@ end
 
 -- make the link flag
 function nf_link(self, lib)
-    if lib:endswith(".a") or lib:endswith(".so") or lib:endswith(".dylib") or lib:endswith(".lib") then
+    if self:is_plat("linux") and (lib:endswith(".a") or lib:endswith(".so")) and not lib:find(path.sep(), 1, true) then
+        return "-l:" .. lib
+    elseif lib:endswith(".a") or lib:endswith(".so") or lib:endswith(".dylib") or lib:endswith(".lib") then
         return lib
     else
         return "-l" .. lib
@@ -300,6 +307,36 @@ function nf_syslink(self, lib)
     return nf_link(self, lib)
 end
 
+-- make the link group flag
+function nf_linkgroup(self, linkgroup, opt)
+    local linkflags = {}
+    for _, lib in ipairs(linkgroup) do
+        table.insert(linkflags, nf_link(self, lib))
+    end
+    local flags = {}
+    local extra = opt.extra
+    if extra and not self:is_plat("macosx", "windows", "mingw") then
+        local group = extra.group
+        local whole = extra.whole
+        if group and whole then
+            -- https://github.com/xmake-io/xmake/issues/4308
+            table.join2(flags, "-Wl,--whole-archive", "-Wl,--start-group", linkflags, "-Wl,--end-group", "-Wl,--no-whole-archive")
+        elseif group then
+            table.join2(flags, "-Wl,--start-group", linkflags, "-Wl,--end-group")
+        elseif whole then
+            table.join2(flags, "-Wl,--whole-archive", linkflags, "-Wl,--no-whole-archive")
+        end
+        local static = extra.static
+        if static then
+            table.join2(flags, "-Wl,-Bstatic", linkflags, "-Wl,-Bdynamic")
+        end
+    end
+    if #flags == 0 then
+        flags = linkflags
+    end
+    return flags
+end
+
 -- make the linkdir flag
 function nf_linkdir(self, dir)
     return {"-L" .. path.translate(dir)}
@@ -381,8 +418,9 @@ function nf_encoding(self, encoding)
 end
 
 -- make the c precompiled header flag
-function nf_pcheader(self, pcheaderfile, target)
+function nf_pcheader(self, pcheaderfile, opt)
     if self:kind() == "cc" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("c")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -393,8 +431,9 @@ function nf_pcheader(self, pcheaderfile, target)
 end
 
 -- make the c++ precompiled header flag
-function nf_pcxxheader(self, pcheaderfile, target)
+function nf_pcxxheader(self, pcheaderfile, opt)
     if self:kind() == "cxx" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("cxx")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -405,8 +444,9 @@ function nf_pcxxheader(self, pcheaderfile, target)
 end
 
 -- make the objc precompiled header flag
-function nf_pmheader(self, pcheaderfile, target)
+function nf_pmheader(self, pcheaderfile, opt)
     if self:kind() == "mm" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("m")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -417,8 +457,9 @@ function nf_pmheader(self, pcheaderfile, target)
 end
 
 -- make the objc++ precompiled header flag
-function nf_pmxxheader(self, pcheaderfile, target)
+function nf_pmxxheader(self, pcheaderfile, opt)
     if self:kind() == "mxx" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("mxx")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -453,8 +494,10 @@ function linkargv(self, objectfiles, targetkind, targetfile, flags, opt)
     -- add rpath for dylib (macho), e.g. -install_name @rpath/file.dylib
     local flags_extra = {}
     if targetkind == "shared" and self:is_plat("macosx", "iphoneos", "watchos") then
-        table.insert(flags_extra, "-install_name")
-        table.insert(flags_extra, "@rpath/" .. path.filename(targetfile))
+        if not table.contains(flags, "-install_name") then
+            table.insert(flags_extra, "-install_name")
+            table.insert(flags_extra, "@rpath/" .. path.filename(targetfile))
+        end
     end
 
     -- add `-Wl,--out-implib,outputdir/libxxx.a` for xxx.dll on mingw/gcc
@@ -717,13 +760,6 @@ function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
     return self:program(), table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
 end
 
--- get modules cache directory
-function _modules_cachedir(target)
-    if target and target.autogendir and target:data("cxx.has_modules") then -- we need to ignore option instance
-        return path.join(target:autogendir(), "rules", "modules", "cache")
-    end
-end
-
 -- make the compile arguments list
 function compargv(self, sourcefile, objectfile, flags)
     -- precompiled header?
@@ -827,7 +863,6 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
                 if depfile and os.isfile(depfile) then
                     if dependinfo then
                         dependinfo.depfiles_gcc = io.readfile(depfile, {continuation = "\\"})
-                        dependinfo.modules_cachedir = _modules_cachedir(opt.target)
                     end
 
                     -- remove the temporary dependent file
