@@ -81,7 +81,7 @@ function _checkout(package, url, sourcedir, opt)
     -- remove temporary directory
     os.rm(sourcedir .. ".tmp")
 
-    -- we need enable longpaths on windows
+    -- we need to enable longpaths on windows
     local longpaths = package:policy("platform.longpaths")
 
     -- download package from branches?
@@ -89,28 +89,47 @@ function _checkout(package, url, sourcedir, opt)
     local branch = package:branch()
     if branch then
 
-        -- we need select the correct default branch
+        -- we need to select the correct default branch
         -- @see https://github.com/xmake-io/xmake/issues/3248
         if branch == "@default" then
             branch = nil
         end
 
-        -- only shadow clone this branch
-        git.clone(url, {depth = 1, recursive = true, longpaths = longpaths, branch = branch, outputdir = packagedir})
+        -- only shallow clone this branch
+        git.clone(url, {depth = 1, recursive = true, shallow_submodules = true, longpaths = longpaths, branch = branch, outputdir = packagedir})
 
     -- download package from revision or tag?
     else
 
-        -- clone whole history and tags
-        git.clone(url, {longpaths = longpaths, outputdir = packagedir})
+        -- get tag and revision?
+        local tag
+        local revision = package:revision(opt.url_alias)
+        if revision and (#revision ~= 40 or not revision:match("%w+")) then
+            tag = revision
+        end
+        if not tag then
+            tag = package:tag()
+        end
+        if not revision then
+            revision = tag or package:commit() or package:version_str()
+        end
 
-        -- attempt to checkout the given version
-        local revision = package:revision(opt.url_alias) or package:tag() or package:commit() or package:version_str()
-        git.checkout(revision, {repodir = packagedir})
+        -- only shallow clone this tag
+        -- @see https://github.com/xmake-io/xmake/issues/4151
+        if tag and git.clone.can_clone_tag() then
+            git.clone(url, {depth = 1, recursive = true, shallow_submodules = true, longpaths = longpaths, branch = tag, outputdir = packagedir})
+        else
 
-        -- update all submodules
-        if os.isfile(path.join(packagedir, ".gitmodules")) then
-            git.submodule.update({init = true, recursive = true, longpaths = longpaths, repodir = packagedir})
+            -- clone whole history and tags
+            git.clone(url, {longpaths = longpaths, outputdir = packagedir})
+
+            -- attempt to checkout the given version
+            git.checkout(revision, {repodir = packagedir})
+
+            -- update all submodules
+            if os.isfile(path.join(packagedir, ".gitmodules")) and opt.url_submodules ~= false then
+                git.submodule.update({init = true, recursive = true, longpaths = longpaths, repodir = packagedir})
+            end
         end
     end
 
@@ -132,7 +151,7 @@ function _download(package, url, sourcedir, opt)
 
     -- get sourcehash from the given url
     --
-    -- we need not sourcehash and skip checksum to try download it directly if no version list in package()
+    -- we don't need sourcehash and skip checksum to try download it directly if no version list in package()
     -- @see https://github.com/xmake-io/xmake/issues/930
     -- https://github.com/xmake-io/xmake/issues/1009
     --
@@ -198,7 +217,7 @@ function _download(package, url, sourcedir, opt)
         local filedirs = os.filedirs(path.join(sourcedir_tmp, "*"))
         if #filedirs == 1 and os.isdir(filedirs[1]) then
             os.mv(filedirs[1], sourcedir)
-            -- we need anchor it to avoid expand it when installing package
+            -- we need to anchor it to avoid expand it when installing package
             io.writefile(path.join(sourcedir, "__sourceroot_anchor__.txt"), "")
             os.rm(sourcedir_tmp)
         else
@@ -212,7 +231,7 @@ function _download(package, url, sourcedir, opt)
         os.mkdir(sourcedir)
         raise("cannot extract %s, maybe missing extractor or invalid package file!", packagefile)
     else
-        -- if it is not archive file, we need only create empty source file and use package:originfile()
+        -- if it is not archive file, we only need to create empty source directory and use package:originfile()
         os.tryrm(sourcedir)
         os.mkdir(sourcedir)
     end
@@ -283,6 +302,7 @@ function main(package)
         local url_alias = package:url_alias(url)
         local url_excludes = package:url_excludes(url)
         local url_http_headers = package:url_http_headers(url)
+        local url_submodules = package:extraconf("urls", url, "submodules")
 
         -- filter url
         url = filter.handle(url, package)
@@ -314,10 +334,12 @@ function main(package)
                         sourcedir = sourcedir,
                         url = url,
                         url_alias = url_alias,
-                        url_excludes = url_excludes})
+                        url_excludes = url_excludes,
+                        url_submodules = url_submodules})
                 elseif git.checkurl(url) then
                     _checkout(package, url, sourcedir, {
-                        url_alias = url_alias})
+                        url_alias = url_alias,
+                        url_submodules = url_submodules})
                 else
                     _download(package, url, sourcedir, {
                         url_alias = url_alias,
