@@ -26,8 +26,9 @@ import("core.platform.platform")
 import("core.package.repository")
 import("devel.git")
 import("net.proxy")
-import("private.async.runjobs")
+import("async.runjobs")
 import("private.action.require.impl.environment")
+import("private.service.remote_build.action", {alias = "remote_build_action"})
 
 function _clear_quick_search_cache(is_global)
     if is_global then
@@ -214,6 +215,11 @@ end
 -- main
 function main()
 
+    -- do action for remote?
+    if remote_build_action.enabled() then
+        return remote_build_action()
+    end
+
     -- load project if operate local repositories
     if not option.get("global") then
         _load_project()
