@@ -44,7 +44,7 @@ end
 -- get stl modules cache directory
 function stlmodules_cachedir(target, opt)
     opt = opt or {}
-    local stlcachedir = path.join(config.buildir(), "stlmodules", "cache", config.mode() or "release")
+    local stlcachedir = path.join(config.buildir(), "stlmodules", "cache", target:arch(), config.mode() or "release")
     if opt.mkdir and not os.isdir(stlcachedir) then
         os.mkdir(stlcachedir)
         os.mkdir(path.join(stlcachedir, "experimental"))
@@ -474,16 +474,12 @@ end
 function find_angle_header_file(target, file)
     local headerpaths = modules_support(target).toolchain_includedirs(target)
     for _, dep in ipairs(target:orderdeps()) do
-        local includedirs = dep:get("sysincludedirs") or dep:get("includedirs")
-        if includedirs then
-            table.join2(headerpaths, includedirs)
-        end
+        local includedirs = table.join(dep:get("sysincludedirs") or {}, dep:get("includedirs") or {})
+        table.join2(headerpaths, includedirs)
     end
     for _, pkg in pairs(target:pkgs()) do
-        local includedirs = pkg:get("sysincludedirs") or pkg:get("includedirs")
-        if includedirs then
-            table.join2(headerpaths, includedirs)
-        end
+        local includedirs = table.join(pkg:get("sysincludedirs") or {}, pkg:get("includedirs") or {})
+        table.join2(headerpaths, includedirs)
     end
     table.join2(headerpaths, target:get("includedirs"))
     local p = find_file(file, headerpaths)
@@ -534,6 +530,9 @@ function fallback_generate_dependencies(target, jsonfile, sourcefile, preprocess
     sourcecode = sourcecode:gsub("//.-\n", "\n")
     sourcecode = sourcecode:gsub("/%*.-%*/", "")
     for _, line in ipairs(sourcecode:split("\n", {plain = true})) do
+        if line:match("#") then
+            goto continue
+        end
         if not module_name_export then
             module_name_export = line:match("export%s+module%s+(.+)%s*;") or line:match("export%s+__preprocessed_module%s+(.+)%s*;")
         end
@@ -544,7 +543,7 @@ function fallback_generate_dependencies(target, jsonfile, sourcefile, preprocess
             end
         end
         local module_depname = line:match("import%s+(.+)%s*;")
-        -- we need parse module interface dep in cxx/impl_unit.cpp, e.g. hello.mpp and hello_impl.cpp
+        -- we need to parse module interface dep in cxx/impl_unit.cpp, e.g. hello.mpp and hello_impl.cpp
         -- @see https://github.com/xmake-io/xmake/pull/2664#issuecomment-1213167314
         if not module_depname and not has_module_extension(sourcefile) then
             module_depname = module_name_private
@@ -572,6 +571,7 @@ function fallback_generate_dependencies(target, jsonfile, sourcefile, preprocess
             table.insert(module_deps, module_dep)
             module_deps_set:insert(module_depname)
         end
+        ::continue::
     end
 
     if module_name_export or internal then
@@ -728,20 +728,10 @@ end
 
 function get_outputdir(target, module)
     local cachedir = modules_cachedir(target)
-    local modulepath = module.path or module
-    local cached = localcache():get2("modules_paths", modulepath)
-    if cached then
-        if not os.exists(cached) then
-            os.mkdir(cached)
-        end
-        return cached
-    else
-        local modulehash = get_modulehash(target, modulepath)
-        local moduledir = path.join(cachedir, modulehash)
-        localcache():set2("modules_paths", modulepath, moduledir)
-        if not os.exists(moduledir) then
-            os.mkdir(moduledir)
-        end
-        return moduledir
+    local modulehash = get_modulehash(target, module.path or module)
+    local outputdir = path.join(cachedir, modulehash)
+    if not os.exists(outputdir) then
+        os.mkdir(outputdir)
     end
+    return outputdir
 end
