@@ -20,6 +20,7 @@
 
 -- imports
 import("core.base.option")
+import("core.base.json")
 import("core.tool.compiler")
 import("core.project.project")
 import("core.project.depend")
@@ -43,14 +44,14 @@ end
 -- /usr/include/c++/11/iostream build/.gens/stl_headerunit/linux/x86_64/release/stlmodules/cache/iostream.gcm
 -- hello build/.gens/stl_headerunit/linux/x86_64/release/rules/modules/cache/hello.gcm
 --
-function _add_module_to_mapper(file, module, bmi)
+function _add_module_to_mapper(file, modulepath, bmi)
     for line in io.lines(file) do
-        if line:startswith(module .. " ") then
+        if line:startswith(modulepath .. " ") then
             return false
         end
     end
     local f = io.open(file, "a")
-    f:print("%s %s", module, bmi)
+    f:print("%s %s", modulepath, bmi)
     f:close()
     return true
 end
@@ -72,8 +73,14 @@ function load(target)
         os.rm(_get_module_mapper(target))
     end
     target:add("cxxflags", {modulesflag, modulemapperflag .. path.translate(_get_module_mapper(target))}, {force = true, expand = false})
-    -- fix cxxabi issue, @see https://github.com/xmake-io/xmake/issues/2716#issuecomment-1225057760
-    target:add("cxxflags", "-D_GLIBCXX_USE_CXX11_ABI=0")
+    -- fix cxxabi issue
+    -- @see https://github.com/xmake-io/xmake/issues/2716#issuecomment-1225057760
+    -- https://github.com/xmake-io/xmake/issues/3855
+    if target:policy("build.c++.gcc.modules.cxx11abi") then
+        target:add("cxxflags", "-D_GLIBCXX_USE_CXX11_ABI=1")
+    else
+        target:add("cxxflags", "-D_GLIBCXX_USE_CXX11_ABI=0")
+    end
 end
 
 -- get includedirs for stl headers
@@ -103,10 +110,10 @@ function _get_toolchain_includedirs_for_stlheaders(includedirs, gcc)
 end
 
 -- do compile for batchcmds
--- @note we need use batchcmds:compilev to translate paths in compflags for generator, e.g. -Ixx
-function _batchcmds_compile(batchcmds, target, flags)
+-- @note we need to use batchcmds:compilev to translate paths in compflags for generator, e.g. -Ixx
+function _batchcmds_compile(batchcmds, target, flags, sourcefile)
     local compinst = target:compiler("cxx")
-    local compflags = compinst:compflags({target = target})
+    local compflags = compinst:compflags({sourcefile = sourcefile, target = target})
     batchcmds:compilev(table.join(compflags or {}, flags), {compiler = compinst, sourcekind = "cxx"})
 end
 
@@ -115,8 +122,8 @@ function _build_modulefile(target, sourcefile, opt)
     local objectfile = opt.objectfile
     local dependfile = opt.dependfile
     local compinst = compiler.load("cxx", {target = target})
-    local compflags = table.join("-x", "c++", compinst:compflags({target = target}))
-    local dependinfo = option.get("rebuild") and {} or (depend.load(dependfile) or {})
+    local compflags = table.join("-x", "c++", compinst:compflags({sourcefile = sourcefile, target = target}))
+    local dependinfo = target:is_rebuilt() and {} or (depend.load(dependfile, {target = target}) or {})
 
     -- need build this object?
     local dryrun = option.get("dry-run")
@@ -169,7 +176,6 @@ end
 
 -- generate dependency files
 function generate_dependencies(target, sourcebatch, opt)
-    local cachedir = common.modules_cachedir(target)
     local compinst = target:compiler("cxx")
     local common_args = {"-E", "-x", "c++"}
     local depformatflag = get_depflag(target, "p1689r5") or get_depflag(target, "trtbd")
@@ -183,17 +189,13 @@ function generate_dependencies(target, sourcebatch, opt)
                 progress.show(opt.progress, "${color.build.object}generating.module.deps %s", sourcefile)
             end
 
-            local outputdir = path.translate(path.join(cachedir, path.directory(path.relative(sourcefile, projectdir))))
-            if not os.isdir(outputdir) then
-                os.mkdir(outputdir)
-            end
-
+            local outputdir = common.get_outputdir(target, sourcefile)
             local jsonfile = path.translate(path.join(outputdir, path.filename(sourcefile) .. ".json"))
-            if depformatflag and depfileflag and depoutputflag then
+            if depformatflag and depfileflag and depoutputflag and not target:policy("build.c++.gcc.fallbackscanner") then
                 local ifile = path.translate(path.join(outputdir, path.filename(sourcefile) .. ".i"))
                 local dfile = path.translate(path.join(outputdir, path.filename(sourcefile) .. ".d"))
                 local args = {sourcefile, "-MT", jsonfile, "-MD", "-MF", dfile, depformatflag, depfileflag .. jsonfile, depoutputflag .. target:objectfile(sourcefile), "-o", ifile}
-                local compflags = compinst:compflags({target = target})
+                local compflags = compinst:compflags({sourcefile = sourcefile, target = target})
                 -- module mapper flag force gcc to check the imports but this is not wanted at this stage
                 local modulemapperflag = get_modulemapperflag(target) .. path.translate(_get_module_mapper(target))
                 table.remove(compflags, table.unpack(table.find(compflags, modulemapperflag)))
@@ -203,28 +205,11 @@ function generate_dependencies(target, sourcebatch, opt)
             else
                 common.fallback_generate_dependencies(target, jsonfile, sourcefile, function(file)
                     local compinst = target:compiler("cxx")
-                    local defines = {}
-                    for _, define in ipairs(target:get("defines")) do
-                        table.insert(defines, "-D" .. define)
-                    end
-                    local includedirs = table.join({}, target:get("includedirs"))
-                    for _, dep in ipairs(target:orderdeps()) do
-                        local includedir = dep:get("sysincludedirs") or dep:get("includedirs")
-                        if includedir then
-                            table.join2(includedirs, includedir)
-                        end
-                    end
-                    for _, pkg in pairs(target:pkgs()) do
-                        local includedir = pkg:get("sysincludedirs") or pkg:get("includedirs")
-                        if includedir then
-                            table.join2(includedirs, includedir)
-                        end
-                    end
-                    for i, includedir in pairs(includedirs) do
-                        includedirs[i] = "-I" .. includedir
-                    end
+                    local compflags = compinst:compflags({sourcefile = file, target = target})
+                    -- exclude -fmodule* flags because, when they are set gcc try to find bmi of imported modules but they don't exists a this point of compilation
+                    table.remove_if(compflags, function(_, flag) return flag:startswith("-fmodule") end)
                     local ifile = path.translate(path.join(outputdir, path.filename(file) .. ".i"))
-                    os.vrunv(compinst:program(), table.join(common_args, includedirs, defines, {get_cppversionflag(target), file,  "-o", ifile}))
+                    os.vrunv(compinst:program(), table.join(common_args, compflags, {file,  "-o", ifile}))
                     local content = io.readfile(ifile)
                     os.rm(ifile)
                     return content
@@ -234,7 +219,7 @@ function generate_dependencies(target, sourcebatch, opt)
 
             local dependinfo = io.readfile(jsonfile)
             return { moduleinfo = dependinfo }
-        end, {dependfile = dependfile, files = {sourcefile}})
+        end, {dependfile = dependfile, files = {sourcefile}, changed = target:is_rebuilt()})
     end
     return changed
 end
@@ -270,7 +255,7 @@ function generate_stl_headerunits_for_batchjobs(target, batchjobs, headerunits,
                     os.vrunv(compinst:program(), flags)
                     _add_module_to_mapper(mapper_file, headerunit.path, path.absolute(bmifile, projectdir))
                     os.tryrm(mapper_file_tmp)
-                end, {dependfile = target:dependfile(bmifile), files = {headerunit.path}})
+                end, {dependfile = target:dependfile(bmifile), files = {headerunit.path}, changed = target:is_rebuilt()})
             end, {rootjob = opt.rootjob})
         else
             _add_module_to_mapper(mapper_file, headerunit.path, path.absolute(bmifile, projectdir))
@@ -304,28 +289,21 @@ end
 function generate_user_headerunits_for_batchjobs(target, batchjobs, headerunits, opt)
     local compinst = target:compiler("cxx")
     local mapper_file = _get_module_mapper(target)
-    local cachedir = common.modules_cachedir(target)
 
     -- build headerunits
     local projectdir = os.projectdir()
     for _, headerunit in ipairs(headerunits) do
-        local file = path.relative(headerunit.path, projectdir)
-        local objectfile = target:objectfile(file)
-        local outputdir
         local headerunit_path
-        if headerunit.type == ":quote" then
-            outputdir = path.join(cachedir, path.directory(path.relative(headerunit.path, projectdir)))
-        else
-            outputdir = path.join(cachedir, path.directory(headerunit.path))
-        end
-        local bmifilename = path.basename(objectfile) .. get_bmi_extension()
-        local bmifile = (outputdir and path.join(outputdir, bmifilename) or bmifilename)
         if headerunit.type == ":quote" then
             headerunit_path = path.join(".", path.relative(headerunit.path, projectdir))
         elseif headerunit.type == ":angle" then
             -- if path is relative then its a subtarget path
             headerunit_path = path.is_absolute(headerunit.path) and headerunit.path or path.join(".", headerunit.path)
         end
+        local objectfile = target:objectfile(headerunit_path)
+        local outputdir = common.get_outputdir(target, headerunit.path)
+        local bmifilename = path.basename(objectfile) .. get_bmi_extension()
+        local bmifile = path.join(outputdir, bmifilename)
         batchjobs:addjob(headerunit.name, function (index, total)
             depend.on_changed(function()
                 progress.show((index * 100) / total, "${color.build.object}compiling.headerunit.$(mode) %s", headerunit.name)
@@ -333,20 +311,27 @@ function generate_user_headerunits_for_batchjobs(target, batchjobs, headerunits,
                 if not os.isdir(objectdir) then
                     os.mkdir(objectdir)
                 end
-                if not os.isdir(outputdir) then
-                    os.mkdir(outputdir)
-                end
 
                 -- generate headerunit
                 local args = { "-c" }
                 if headerunit.type == ":quote" then
-                    table.join2(args, { "-I", path.directory(path.relative(headerunit.path, projectdir)), "-x", "c++-user-header", headerunit.name })
+                    local includedir
+                    local p = headerunit.path
+                    if p:endswith(headerunit.name) then
+                        includedir = p:sub(1, #p - #headerunit.name - 1)
+                    else
+                        includedir = path.directory(p)
+                    end
+                    if path.is_absolute(includedir) then
+                        includedir = path.relative(includedir, projectdir)
+                    end
+                    table.join2(args, { "-I", includedir, "-x", "c++-user-header", headerunit.name })
                 elseif headerunit.type == ":angle" then
                     table.join2(args, { "-x", "c++-system-header", headerunit.name })
                 end
                 os.vrunv(compinst:program(), table.join(compinst:compflags({target = target}), args))
 
-            end, {dependfile = target:dependfile(bmifile), files = {headerunit.path}})
+            end, {dependfile = target:dependfile(bmifile), files = {headerunit.path}, changed = target:is_rebuilt()})
         end, {rootjob = opt.rootjob})
         _add_module_to_mapper(mapper_file, headerunit_path, path.absolute(bmifile, projectdir))
     end
@@ -355,36 +340,38 @@ end
 -- generate target user header units for batchcmds
 function generate_user_headerunits_for_batchcmds(target, batchcmds, headerunits, opt)
     local mapper_file = _get_module_mapper(target)
-    local cachedir = common.modules_cachedir(target)
 
     -- build headerunits
     local projectdir = os.projectdir()
     local depmtime = 0
     for _, headerunit in ipairs(headerunits) do
-        local file = path.relative(headerunit.path, projectdir)
-        local objectfile = target:objectfile(file)
-        local outputdir
-        if headerunit.type == ":quote" then
-            outputdir = path.join(cachedir, path.directory(path.relative(headerunit.path, projectdir)))
-        else
-            outputdir = path.join(cachedir, path.directory(headerunit.path))
-        end
-        batchcmds:mkdir(outputdir)
-
-        local bmifilename = path.basename(objectfile) .. get_bmi_extension()
-        local bmifile = (outputdir and path.join(outputdir, bmifilename) or bmifilename)
-        batchcmds:mkdir(path.directory(objectfile))
-
         local flags = {"-c"}
         local headerunit_path
         if headerunit.type == ":quote" then
-            table.join2(flags, {"-I", path(path.relative(headerunit.path, projectdir)):directory(), "-x", "c++-user-header", headerunit.name})
+            local includedir
+            local p = headerunit.path
+            if p:endswith(headerunit.name) then
+                includedir = p:sub(1, #p - #headerunit.name - 1)
+            else
+                includedir = path.directory(p)
+            end
+            if path.is_absolute(includedir) then
+                includedir = path.relative(includedir, projectdir)
+            end
+            table.join2(flags, {"-I", path(includedir), "-x", "c++-user-header", headerunit.name})
             headerunit_path = path.join(".", path.relative(headerunit.path, projectdir))
         elseif headerunit.type == ":angle" then
             table.join2(flags, {"-x", "c++-system-header", headerunit.name})
             -- if path is relative then its a subtarget path
             headerunit_path = path.is_absolute(headerunit.path) and headerunit.path or path.join(".", headerunit.path)
         end
+        local objectfile = target:objectfile(headerunit_path)
+        local outputdir = common.get_outputdir(target, headerunit.path)
+        batchcmds:mkdir(outputdir)
+
+        local bmifilename = path.basename(objectfile) .. get_bmi_extension()
+        local bmifile = (outputdir and path.join(outputdir, bmifilename) or bmifilename)
+        batchcmds:mkdir(path.directory(objectfile))
 
         batchcmds:show_progress(opt.progress, "${color.build.object}compiling.headerunit.$(mode) %s", headerunit.name)
         _batchcmds_compile(batchcmds, target, flags)
@@ -424,6 +411,21 @@ function build_modules_for_batchjobs(target, batchjobs, objectfiles, modules, op
             end
             local moduleinfo = table.copy(provide) or {}
             local dependfile = (provide and provide.bmi) and target:dependfile(provide.bmi) or target:dependfile(objectfile)
+
+            if provide then
+                local fileconfig = target:fileconfig(cppfile)
+                if fileconfig and fileconfig.install then
+                    batchjobs:addjob(name .. "_metafile", function(index, total)
+                        local metafilepath = common.get_metafile(target, cppfile)
+                        depend.on_changed(function()
+                            progress.show((index * 100) / total, "${color.build.object}generating.module.metadata %s", name)
+                            local metadata = common.generate_meta_module_info(target, name, cppfile, module.requires)
+                            json.savefile(metafilepath, metadata)
+                        end, {dependfile = target:dependfile(metafilepath), files = {cppfile}, changed = target:is_rebuilt()})
+                    end, {rootjob = opt.rootjob})
+                end
+            end
+
             table.join2(moduleinfo, {
                 name = name or cppfile,
                 deps = table.keys(module.requires or {}),
@@ -445,11 +447,16 @@ function build_modules_for_batchjobs(target, batchjobs, objectfiles, modules, op
                     end
 
                     if provide or common.has_module_extension(cppfile) then
-                        _build_modulefile(target, cppfile, {
-                            objectfile = objectfile,
-                            dependfile = dependfile,
-                            name = name or cppfile,
-                            progress = (index * 100) / total})
+                        if not common.memcache():get2(name or cppfile, "compiling") then
+                            if name and module.external then
+                                common.memcache():set2(name or cppfile, "compiling", true)
+                            end
+                            _build_modulefile(target, cppfile, {
+                                objectfile = objectfile,
+                                dependfile = dependfile,
+                                name = name or cppfile,
+                                progress = (index * 100) / total})
+                        end
                         target:add("objectfiles", objectfile)
                     end
                 end)})
@@ -504,13 +511,13 @@ function build_modules_for_batchcmds(target, batchcmds, objectfiles, modules, op
                     break
                 end
             end
-            local flags = {"-x", "c++", "-c", path(cppfile), "-o", path(objectfile)}
-            batchcmds:show_progress(opt.progress, "${color.build.object}compiling.module.$(mode) %s", name or cppfile)
-            batchcmds:mkdir(path.directory(objectfile))
-            _batchcmds_compile(batchcmds, target, flags)
-            batchcmds:add_depfiles(cppfile)
-            target:add("objectfiles", objectfile)
-            if provide then
+            if provide or common.has_module_extension(cppfile) then
+                local flags = {"-x", "c++", "-c", path(cppfile), "-o", path(objectfile)}
+                batchcmds:show_progress(opt.progress, "${color.build.object}compiling.module.$(mode) %s", name or cppfile)
+                batchcmds:mkdir(path.directory(objectfile))
+                _batchcmds_compile(batchcmds, target, flags, cppfile)
+                batchcmds:add_depfiles(cppfile)
+                target:add("objectfiles", objectfile)
                 _add_module_to_mapper(mapper_file, name, path.absolute(provide.bmi, projectdir))
             end
             depmtime = math.max(depmtime, os.mtime(provide and provide.bmi or objectfile))
